/******/ (() => { // webpackBootstrap
/******/ 	"use strict";
var __webpack_exports__ = {};

handleDocumentReady();
// Initialize variables at the top to prevent "not defined" errors
let currentAction = null;
let actionInProgress = false;
const activeActions = new Map();
let totalMutations = 0;
let lastGoogleAuthAccountId = null;
let googleOAuthRequestToken = null;
// Constants for reactive timing
const TIMING = {
    // Maximum time to wait for any action
    MAX_TIMEOUT: 25000,
    // Time to wait for DOM mutations to settle
    MUTATION_SETTLE: 300,
    // Minimum time between actions (for stability)
    MIN_ACTION_SPACING: 2000,
    // Action execution minimums (to ensure stability)
    ACTION_MINIMUMS: {
        type: 8000,
        click: 5000,
        navigate: 10000,
        findVal: 8000,
    },
    // DOM refresh timeout
    DOM_REFRESH_TIMEOUT: 2000,
};
// Original timing constants (kept for compatibility)
const INITIAL_DELAY = 1000;
const ACTION_TIMEOUT = 25000;
const VERIFICATION_INTERVAL = 500;
// Track script state
let lastActionTime = Date.now();
let sequenceState = null;
// Add this at the top with other state variables
let googleOAuthTabId = null;
// Add DOM stability tracking variables
let lastDomMutation = 0;
let domStabilityObserver = null;
let networkRequestsInProgress = 0;
let lastNetworkActivity = 0;
let urlChangeObserver = null;
let lastUrlCheckTime = 0;
const URL_CHECK_THROTTLE = 500; // Only check URL changes every 250ms
// Debounce map to prevent duplicate value updates
const valueUpdateDebounce = new Map();
// Cache for interactability checks to avoid redundant expensive operations
const interactabilityCache = new Map();
// Cache TTL in milliseconds (500ms)
const INTERACTABILITY_CACHE_TTL = 500;
// Cache for click operations to avoid redundant searches and clicks
const clickCache = new Map();
// Click cache TTL in milliseconds (5 seconds)
const CLICK_CACHE_TTL = 5000;
// Add a helper function to refresh the DOM
async function refreshDOM(timeout = TIMING.DOM_REFRESH_TIMEOUT) {
    console.log(`🔄 Refreshing DOM state (timeout: ${timeout}ms)`);
    return new Promise((resolve) => {
        // Create a mutation observer to detect DOM changes
        const observer = new MutationObserver((mutations) => {
            console.log(`🔄 DOM mutations detected: ${mutations.length}`);
        });
        // Start observing
        observer.observe(document.body, {
            childList: true,
            subtree: true,
            attributes: true,
            characterData: true,
        });
        // Set a timeout to resolve after specified time
        setTimeout(() => {
            observer.disconnect();
            console.log(`✅ DOM refresh complete after ${timeout}ms`);
            resolve(true);
        }, timeout);
    });
}
// Modify executeAction to remove waitForDOMStability call
async function executeAction(actionDetails) {
    const actionType = actionDetails.action;
    console.log(`🔄 Executing action: ${actionType}`, actionDetails);
    try {
        // Execute the action based on type
        switch (actionType) {
            case "navigate":
                return await executeNavigateAction(actionDetails);
            case "click":
                return await executeClickAction(actionDetails);
            case "type":
                return await executeTypeAction(actionDetails);
            case "findVal":
                return await executeFindValAction(actionDetails);
            default:
                console.error(`❌ Unknown action type: ${actionType}`);
                return {
                    status: "failed",
                    success: false,
                    error: `Unknown action type: ${actionType}`,
                };
        }
    }
    catch (error) {
        console.error(`❌ Error executing action ${actionType}:`, error);
        return {
            status: "failed",
            success: false,
            error: String(error),
        };
    }
}
/**
 * Handle document ready event and process Google OAuth if needed
 */
async function handleDocumentReady() {
    // Only handle Google OAuth if we're on a Google accounts page
    if (window.location.href.includes("accounts.google.com")) {
        // Check if this is an OAuth flow initiated by our extension
        const isOurOAuthFlow = await checkIfOurOAuthFlow();
        if (isOurOAuthFlow) {
            console.log("🔄 Our extension's Google OAuth popup detected");
            await handleGoogleOAuth();
        }
        else {
            console.log("ℹ️ Google page detected, but not our OAuth flow - ignoring");
        }
    }
    // Other document ready logic can go here
}
/**
 * Check if the current OAuth flow was initiated by our extension
 */
async function checkIfOurOAuthFlow() {
    try {
        if (typeof chrome !== "undefined" &&
            chrome.storage &&
            chrome.storage.local) {
            // Get both the account ID and request token
            const data = await chrome.storage.local.get([
                "lastGoogleAuthAccountId",
                "googleOAuthRequestToken",
                "googleOAuthInitiatedAt",
            ]);
            lastGoogleAuthAccountId = data.lastGoogleAuthAccountId || null;
            googleOAuthRequestToken = data.googleOAuthRequestToken || null;
            // Check if we have a valid token and it's not too old (15 minute timeout)
            const initiatedAt = data.googleOAuthInitiatedAt || 0;
            const isTokenValid = !!googleOAuthRequestToken;
            const isTokenFresh = Date.now() - initiatedAt < 15 * 60 * 1000; // 15 minutes
            // Only proceed if we have a valid, fresh token and an account ID
            if (lastGoogleAuthAccountId && isTokenValid && isTokenFresh) {
                console.log("✅ Valid OAuth flow detected for account:", lastGoogleAuthAccountId);
                return true;
            }
        }
        return false;
    }
    catch (error) {
        console.error("Error checking OAuth flow:", error);
        return false;
    }
}
/**
 * Handle Google OAuth authentication flow
 */
async function handleGoogleOAuth() {
    try {
        // If we're on the consent page, handle it
        if (window.location.href.includes("signin/oauth")) {
            await handleGoogleConsentPage();
        }
        else {
            // Otherwise try to select the correct account
            await selectGoogleAccount();
        }
    }
    catch (error) {
        console.error("Error in Google OAuth handling:", error);
    }
    finally {
        // Reset action state to prevent blocking
        currentAction = null;
        actionInProgress = false;
    }
}
/**
 * Try to select the correct Google account
 */
async function selectGoogleAccount() {
    if (!lastGoogleAuthAccountId) {
        return;
    }
    try {
        // Wait a moment for the page to stabilize
        await new Promise((resolve) => setTimeout(resolve, 500));
        // Try to click the account with the matching email
        await executeClickAction({
            action: "click",
            textContents: "UserKeys",
            element: "div",
            duration: 1000,
            clickDelay: 69,
        });
    }
    catch (clickError) {
        console.log("Error executing click action:", clickError);
    }
}
/**
 * Handle the Google consent page
 */
async function handleGoogleConsentPage() {
    console.log("🔍 Detected consent page");
    try {
        // Add a small delay to ensure the button is fully rendered and interactive
        await new Promise((resolve) => setTimeout(resolve, 1000));
        // Try to click the continue button
        const result = await executeClickAction({
            action: "click",
            element: "span",
            textContents: "Continue",
            duration: 2000,
        });
        // If the action was successful, send completion signal
        if (result.success) {
            console.log("✅ Continue button clicked successfully");
            await sendOAuthCompletionSignal();
        }
        else {
            console.log("⚠️ Continue button click failed");
        }
    }
    catch (clickError) {
        console.log("Error clicking continue button:", clickError);
    }
}
/**
 * Send a message that OAuth flow is complete
 */
async function sendOAuthCompletionSignal() {
    try {
        // Send the completion message
        chrome.runtime.sendMessage({
            type: "ACTION_STATUS",
            status: "completed",
            success: true,
            action: "googleOAuthComplete",
            accountId: lastGoogleAuthAccountId,
            requestToken: googleOAuthRequestToken,
            completedAt: Date.now(),
        });
        console.log("✅ Sent OAuth completion signal");
        // Clean up the tokens
        await chrome.storage.local.remove([
            "googleOAuthRequestToken",
            "googleOAuthInitiatedAt",
        ]);
    }
    catch (error) {
        console.log("❌ Error sending completion signal:", error);
    }
}
// Add new action execution function
async function executeFindValAction(actionDetails) {
    console.log(`🔍 Starting findVal action for ${actionDetails.slug} with details:`, actionDetails);
    try {
        console.log(`🔄 Calling universalFindText for ${actionDetails.slug}`);
        const result = await universalFindText(actionDetails);
        console.log(`📊 universalFindText returned result for ${actionDetails.slug}:`, result);
        // If we found a valid value, consider it a success regardless of message port status
        if (result !== null) {
            console.log(`✅ Value extraction succeeded for ${actionDetails.slug}: ${result}`);
            // Try to get the current stored value for comparison
            try {
                if (typeof chrome !== "undefined" &&
                    chrome.storage &&
                    chrome.storage.local) {
                    const { [`value_for_${actionDetails.slug}`]: existingValue } = await chrome.storage.local.get(`value_for_${actionDetails.slug}`);
                    if (existingValue !== undefined) {
                        console.log(`📊 Final value comparison for ${actionDetails.slug}:`);
                        console.log(`   - Existing value in system: ${existingValue}`);
                        console.log(`   - Newly extracted value: ${result}`);
                        // Calculate difference
                        const difference = Math.abs(existingValue - result);
                        const percentChange = existingValue
                            ? (difference / existingValue) * 100
                            : 0;
                        console.log(`   - Absolute difference: ${difference.toFixed(4)}`);
                        console.log(`   - Percent change: ${percentChange.toFixed(2)}%`);
                    }
                }
            }
            catch (error) {
                console.warn(`⚠️ Error checking final existing value: ${error}`);
            }
            return {
                status: "completed",
                success: true,
                error: null,
                data: result,
            };
        }
        console.warn(`❌ Value extraction failed for ${actionDetails.slug} - no valid value found`);
        return {
            status: "completed",
            success: false,
            error: "No valid value found or value update failed",
            data: null,
        };
    }
    catch (error) {
        console.error(`❌ Error in findVal action for ${actionDetails.slug}:`, error);
        return {
            status: "completed",
            success: false,
            error: String(error),
            data: null,
        };
    }
}
// Optimized function to check element interactability with caching
function isElementInteractable(element) {
    if (!element || !(element instanceof HTMLElement))
        return false;
    // Check cache first
    const now = Date.now();
    const cached = interactabilityCache.get(element);
    if (cached && now - cached.timestamp < INTERACTABILITY_CACHE_TTL) {
        return cached.result;
    }
    // Fast checks first (these don't cause reflow/repaint)
    if (element.hasAttribute("disabled") ||
        element.getAttribute("aria-disabled") === "true" ||
        element.hasAttribute("hidden") ||
        element.getAttribute("aria-hidden") === "true") {
        // Cache and return result
        interactabilityCache.set(element, { result: false, timestamp: now });
        return false;
    }
    // More expensive checks that cause reflow
    const style = window.getComputedStyle(element);
    if (style.display === "none" ||
        style.visibility === "hidden" ||
        style.opacity === "0") {
        // Cache and return result
        interactabilityCache.set(element, { result: false, timestamp: now });
        return false;
    }
    // Check dimensions (causes reflow)
    const rect = element.getBoundingClientRect();
    if (rect.width === 0 || rect.height === 0) {
        // Cache and return result
        interactabilityCache.set(element, { result: false, timestamp: now });
        return false;
    }
    // Check if element is disabled or readonly for input elements
    if (element instanceof HTMLInputElement ||
        element instanceof HTMLTextAreaElement) {
        if (element.disabled || element.readOnly) {
            // Cache and return result
            interactabilityCache.set(element, { result: false, timestamp: now });
            return false;
        }
    }
    // Most expensive check - elementFromPoint (causes hit testing)
    // Only do this check if the element is in the viewport
    if (rect.bottom >= 0 &&
        rect.top <= window.innerHeight &&
        rect.right >= 0 &&
        rect.left <= window.innerWidth) {
        const centerX = rect.left + rect.width / 2;
        const centerY = rect.top + rect.height / 2;
        const elementAtPoint = document.elementFromPoint(centerX, centerY);
        if (!element.contains(elementAtPoint) &&
            !elementAtPoint?.contains(element)) {
            // Cache and return result
            interactabilityCache.set(element, { result: false, timestamp: now });
            return false;
        }
    }
    // Element passed all checks
    // Cache and return result
    interactabilityCache.set(element, { result: true, timestamp: now });
    return true;
}
// Keep utility functions that are used by executeAction
// Keep utility functions that are used by executeAction
async function universalClicker(options) {
    const { accountId, element, textContents, attribute, selectorText, parentElement, parentAttribute, parentSelectorText, domCheck = false, domRefreshTimeout = TIMING.DOM_REFRESH_TIMEOUT, clickDelay = 100, usePointerEvents = false, // new flag to toggle pointer vs mouse events
     } = options;
    console.log(`🔍 Starting click operation for ${accountId || "unknown"}`);
    if (domCheck) {
        console.log("🔄 Refreshing DOM state before clicking");
        await refreshDOM(domRefreshTimeout);
    }
    const finalTextContents = await replaceTextContentsIfEmail(accountId, textContents);
    async function simulateHumanClick(element) {
        if (!(element instanceof HTMLElement)) {
            console.warn("⚠️ Element is not an HTMLElement, cannot click");
            return false;
        }
        try {
            const rect = element.getBoundingClientRect();
            const centerX = rect.left + rect.width / 2;
            const centerY = rect.top + rect.height / 2;
            const maxXJitter = Math.max(3, rect.width * 0.2);
            const maxYJitter = Math.max(3, rect.height * 0.2);
            const targetX = centerX + (Math.random() * 2 - 1) * maxXJitter;
            const targetY = centerY + (Math.random() * 2 - 1) * maxYJitter;
            const pathPoints = [
                {
                    x: targetX - (60 + Math.random() * 40),
                    y: targetY - (30 + Math.random() * 60),
                },
                {
                    x: targetX - (15 + Math.random() * 25),
                    y: targetY - (5 + Math.random() * 15),
                },
                { x: targetX, y: targetY },
            ];
            for (let i = 0; i < pathPoints.length; i++) {
                const point = pathPoints[i];
                element.dispatchEvent(new MouseEvent("mousemove", {
                    bubbles: true,
                    cancelable: true,
                    view: window,
                    clientX: point.x,
                    clientY: point.y,
                }));
                const baseDelay = (pathPoints.length - i) * 30;
                const jitter = Math.random() * 40;
                await new Promise((resolve) => setTimeout(resolve, baseDelay + jitter));
            }
            if (usePointerEvents) {
                element.dispatchEvent(new PointerEvent("pointerover", {
                    bubbles: true,
                    cancelable: true,
                    clientX: targetX,
                    clientY: targetY,
                    pointerType: "mouse",
                    isPrimary: true,
                }));
                element.dispatchEvent(new PointerEvent("pointermove", {
                    bubbles: true,
                    cancelable: true,
                    clientX: targetX,
                    clientY: targetY,
                    pointerType: "mouse",
                    isPrimary: true,
                }));
                await new Promise((resolve) => setTimeout(resolve, 20 + Math.random() * 40));
                element.dispatchEvent(new PointerEvent("pointerdown", {
                    bubbles: true,
                    cancelable: true,
                    clientX: targetX,
                    clientY: targetY,
                    pointerType: "mouse",
                    isPrimary: true,
                    button: 0,
                }));
                const downUpDelay = Math.floor(Math.random() * 110) + 40;
                await new Promise((resolve) => setTimeout(resolve, downUpDelay));
                element.dispatchEvent(new PointerEvent("pointerup", {
                    bubbles: true,
                    cancelable: true,
                    clientX: targetX,
                    clientY: targetY,
                    pointerType: "mouse",
                    isPrimary: true,
                    button: 0,
                }));
            }
            else {
                element.dispatchEvent(new MouseEvent("mouseover", {
                    bubbles: true,
                    cancelable: true,
                    view: window,
                    clientX: targetX,
                    clientY: targetY,
                }));
                await new Promise((resolve) => setTimeout(resolve, 20 + Math.random() * 40));
                element.dispatchEvent(new MouseEvent("mousedown", {
                    bubbles: true,
                    cancelable: true,
                    view: window,
                    clientX: targetX,
                    clientY: targetY,
                    button: 0,
                }));
                const downUpDelay = Math.floor(Math.random() * 110) + 40;
                await new Promise((resolve) => setTimeout(resolve, downUpDelay));
                element.dispatchEvent(new MouseEvent("mouseup", {
                    bubbles: true,
                    cancelable: true,
                    view: window,
                    clientX: targetX,
                    clientY: targetY,
                    button: 0,
                }));
            }
            const upClickDelay = Math.floor(Math.random() * 10) + 20;
            await new Promise((resolve) => setTimeout(resolve, upClickDelay));
            element.dispatchEvent(new MouseEvent("click", {
                bubbles: true,
                cancelable: true,
                view: window,
                clientX: targetX,
                clientY: targetY,
                button: 0,
            }));
            console.log(`✅ Clicked element: ${element.tagName}${element.id ? "#" + element.id : ""} at (${targetX.toFixed(1)}, ${targetY.toFixed(1)}) (delays: mousedown→mouseup: ${usePointerEvents ? "(pointer)" : "(mouse)"}, mouseup→click: ${upClickDelay}ms)`);
            return true;
        }
        catch (error) {
            console.error("❌ Error simulating click:", error);
            return false;
        }
    }
    let childSelector = "";
    if (element && attribute && selectorText) {
        childSelector = `${element}[${attribute}*="${selectorText}"]`;
    }
    else if (element && attribute) {
        childSelector = `${element}[${attribute}]`;
    }
    else if (element) {
        childSelector = element;
    }
    else {
        childSelector = "*";
    }
    function filterElementsByTextContent(elements) {
        if (!finalTextContents)
            return elements;
        return elements.filter((el) => {
            const text = el.textContent?.trim().toLowerCase() || "";
            return text.includes(finalTextContents.trim().toLowerCase());
        });
    }
    let allElements = [];
    if (parentElement && parentAttribute && parentSelectorText) {
        const parentSelector = `${parentElement}[${parentAttribute}*="${parentSelectorText}"]`;
        console.log(`🔍 Using parent-child selector: "${parentSelector}" > ${childSelector}`);
        const parents = document.querySelectorAll(parentSelector);
        console.log(`📊 Found ${parents.length} parent elements`);
        if (parents.length > 0) {
            for (const parent of parents) {
                const children = parent.querySelectorAll(childSelector);
                allElements = [...allElements, ...Array.from(children)];
            }
        }
    }
    else {
        console.log(`🔍 Using direct selector: "${childSelector}"`);
        allElements = Array.from(document.querySelectorAll(childSelector));
    }
    console.log(`📊 Found ${allElements.length} total elements`);
    const filteredElements = filterElementsByTextContent(allElements);
    console.log(`📊 Found ${filteredElements.length} elements after text filtering`);
    if (filteredElements.length === 0) {
        console.log("⚠️ No elements found matching the criteria");
        return {
            elements: [],
            clickSuccess: false,
            domChecked: domCheck,
        };
    }
    const interactableElements = filteredElements.filter((el) => isElementInteractable(el));
    console.log(`📊 Found ${interactableElements.length} interactable elements`);
    const elementsToClick = interactableElements.length > 0 ? interactableElements : filteredElements;
    const maxElementsToClick = Math.min(elementsToClick.length, 25);
    console.log(`🔍 Will attempt to click ${maxElementsToClick} elements`);
    let clickedCount = 0;
    for (let i = 0; i < maxElementsToClick; i++) {
        const el = elementsToClick[i];
        console.log(`🔍 Attempting to click element ${i + 1}/${maxElementsToClick}`);
        const success = await simulateHumanClick(el);
        if (success) {
            clickedCount++;
            if (i < maxElementsToClick - 1) {
                await new Promise((resolve) => setTimeout(resolve, clickDelay));
            }
        }
    }
    console.log(`✅ Successfully clicked ${clickedCount} out of ${maxElementsToClick} elements`);
    return {
        elements: filteredElements,
        clickSuccess: clickedCount > 0,
        domChecked: domCheck,
        clickedCount: clickedCount,
        totalElements: elementsToClick.length,
    };
}
// Function to replace text contents with UserKeys value in case of Google or FB Auth
async function replaceTextContentsIfEmail(accountId, textContents) {
    if (textContents === "UserKeys") {
        console.log(`Replacing textContents with UserKeys for account ${accountId}`);
        console.log("textContents", textContents);
        const keys = await getStorageData([`UserKeys_${accountId}`]);
        const userKey = keys[`UserKeys_${accountId}`];
        console.log("userKey", userKey);
        if (userKey) {
            return userKey;
        }
        else {
            console.error(`No UserKeys found for account ${accountId}`);
            return null;
        }
    }
    return textContents;
}
// Individual action execution functions
async function executeTypeAction(actionDetails) {
    try {
        console.log(`🔤 Executing TYPE action`);
        // Get credentials for the specified slug
        const { slug, userSelector, passSelector } = actionDetails;
        if (!slug) {
            throw new Error("No slug provided for TYPE action");
        }
        if (!userSelector || !passSelector) {
            throw new Error("Both userSelector and passSelector must be provided");
        }
        // Try different storage formats for credentials
        const storageKeys = [
            `UserKeys_${slug}`, // Old format: username as string
            `PassKeys_${slug}`, // Old format: password as string
        ];
        const storageData = await getStorageData(storageKeys);
        let username = "";
        let password = "";
        if (!username && storageData[`UserKeys_${slug}`]) {
            username = storageData[`UserKeys_${slug}`];
        }
        if (!password && storageData[`PassKeys_${slug}`]) {
            password = storageData[`PassKeys_${slug}`];
        }
        if (!username || !password) {
            throw new Error(`No credentials found for slug: ${slug}`);
        }
        console.log(`🔑 Found credentials for ${slug}`);
        // Input the credentials
        const success = await inputCredentials(userSelector, username, passSelector, password);
        if (success) {
            console.log(`✅ Successfully entered credentials for ${slug}`);
            return {
                status: "completed",
                success: true,
                data: { message: "Credentials entered successfully" },
            };
        }
        else {
            console.warn(`⚠️ Failed to input credentials for ${slug}`);
            return {
                status: "completed",
                success: false,
                error: "Failed to input credentials",
            };
        }
    }
    catch (error) {
        console.error("❌ Error in executeTypeAction:", error);
        return {
            status: "completed",
            success: false,
            error: error instanceof Error ? error.message : String(error),
        };
    }
}
/**
 * Input credentials into login form by typing each letter one at a time
 * @param userSelector Selector for username field
 * @param userText Username text
 * @param passSelector Selector for password field
 * @param passText Password text
 * @returns Promise that resolves to true if successful
 */
async function inputCredentials(userSelector, userText, passSelector, passText) {
    try {
        console.log(`🔑 Inputting credentials`);
        // Find the input fields
        const userInput = document.querySelector(userSelector);
        if (!userInput) {
            console.warn(`⚠️ Username input field not found with selector: ${userSelector}`);
            return false;
        }
        const passInput = document.querySelector(passSelector);
        if (!passInput) {
            console.warn(`⚠️ Password input field not found with selector: ${passSelector}`);
            return false;
        }
        // Type username character by character
        console.log(`🔑 Typing username: ${userText.substring(0, 3)}...`);
        await typeCharByChar(userInput, userText);
        // Small delay between username and password
        await new Promise((resolve) => setTimeout(resolve, 500));
        // Type password character by character
        console.log(`🔑 Typing password: ***`);
        await typeCharByChar(passInput, passText);
        return true;
    }
    catch (error) {
        console.error("❌ Error during credential input:", error);
        return false;
    }
}
/**
 * Types text into an input field one character at a time
 * @param inputElement The input element to type into
 * @param text The text to type
 */
async function typeCharByChar(inputElement, text) {
    // Clear and focus on the field
    inputElement.value = "";
    inputElement.focus();
    inputElement.click();
    // Type each character with a random delay
    let currentText = "";
    for (let i = 0; i < text.length; i++) {
        // Random delay between 50-150ms between keystrokes
        const delay = Math.floor(Math.random() * 100) + 50;
        await new Promise((resolve) => setTimeout(resolve, delay));
        // Add the next character
        currentText += text[i];
        inputElement.value = currentText;
        // Dispatch input event for each character
        inputElement.dispatchEvent(new Event("input", { bubbles: true }));
        // Simulate keydown and keyup events for more realism
        const keyCode = text.charCodeAt(i);
        const keydownEvent = new KeyboardEvent("keydown", {
            key: text[i],
            code: `Key${text[i].toUpperCase()}`,
            keyCode: keyCode,
            which: keyCode,
            bubbles: true,
        });
        const keyupEvent = new KeyboardEvent("keyup", {
            key: text[i],
            code: `Key${text[i].toUpperCase()}`,
            keyCode: keyCode,
            which: keyCode,
            bubbles: true,
        });
        inputElement.dispatchEvent(keydownEvent);
        inputElement.dispatchEvent(keyupEvent);
    }
    // Dispatch change event after typing is complete
    inputElement.dispatchEvent(new Event("change", { bubbles: true }));
    // Small delay after typing is complete
    await new Promise((resolve) => setTimeout(resolve, 300));
}
// Modify the executeClickAction function to use our improved universalClicker function without the fallback logic
async function executeClickAction(actionDetails) {
    try {
        console.log("🖱️ Executing click action:", actionDetails);
        // Handle Google auth button clicks
        if (actionDetails.textContents?.toLowerCase() === "google" &&
            actionDetails.accountId) {
            // console.log(
            //   "📝 Setting up Google auth for account:",
            //   actionDetails.accountId
            // );
            // Generate a unique token for this OAuth flow
            const requestToken = generateUniqueToken();
            // Store both the account ID and the request token
            await chrome.storage.local.set({
                lastGoogleAuthAccountId: actionDetails.accountId,
                googleOAuthRequestToken: requestToken,
                googleOAuthInitiatedAt: Date.now(),
                googleOAuthTabId: null,
            });
        }
        // Prepare universal clicker options
        const universalClickerOptions = {
            accountId: actionDetails.accountId,
            element: actionDetails.element,
            textContents: actionDetails.textContents,
            attribute: actionDetails.attribute,
            selectorText: actionDetails.selectorText,
            parentElement: actionDetails.parentElement,
            parentAttribute: actionDetails.parentAttribute,
            parentSelectorText: actionDetails.parentSelectorText,
            skipCache: actionDetails.skipCache,
            domCheck: actionDetails.domCheck || false, // Always refresh DOM before clicking for better reliability
            domRefreshTimeout: actionDetails.domRefreshTimeout || TIMING.DOM_REFRESH_TIMEOUT,
            clickDelay: actionDetails.clickDelay || 100,
        };
        // Wait for the page to be interactive before attempting to click
        if (document.readyState !== "complete") {
            console.log("⏳ Waiting for document to be ready before clicking");
            await new Promise((resolve) => {
                if (document.readyState === "complete") {
                    resolve(null);
                }
                else {
                    window.addEventListener("load", () => resolve(null), { once: true });
                }
            });
        }
        // Execute the click using universal clicker
        const clickResult = await universalClicker(universalClickerOptions);
        // Check if elements were found
        if (clickResult.elements.length === 0) {
            console.warn("⚠️ No elements found matching the criteria");
            return {
                status: "completed",
                success: false,
                error: "No elements found matching the criteria",
            };
        }
        // Check if click was successful
        if (clickResult.clickSuccess) {
            console.log(`✅ Successfully clicked ${clickResult.clickedCount} element(s)`);
            return {
                status: "completed",
                success: true,
                error: null,
                data: {
                    elementsFound: clickResult.elements.length,
                    clickedCount: clickResult.clickedCount,
                },
            };
        }
        else {
            console.warn("⚠️ Found elements but click was not successful");
            return {
                status: "completed",
                success: false,
                error: "Found elements but click was not successful",
                data: { elementsFound: clickResult.elements.length },
            };
        }
    }
    catch (error) {
        console.error("❌ Error executing click action:", error);
        return {
            status: "failed",
            success: false,
            error: String(error),
        };
    }
}
/**
 * Generate a unique token for OAuth requests
 */
function generateUniqueToken() {
    return "oauth_" + Date.now() + "_" + Math.random().toString(36).substr(2, 9);
}
async function executeNavigateAction(actionDetails) {
    try {
        if (!actionDetails.URL || actionDetails.URL === "undefined") {
            console.error("❌ Invalid URL provided:", actionDetails.URL);
            throw new Error("Invalid URL provided: " + actionDetails.URL);
        }
        const startUrl = window.location.href;
        console.log(`🌐 Navigating from ${startUrl} to ${actionDetails.URL}`);
        // Add a small delay before navigation
        await new Promise((r) => setTimeout(r, 1000));
        // Use location.assign instead of location.href for more reliable navigation
        window.location.assign(actionDetails.URL);
        console.log(`🌐 Navigation initiated to ${actionDetails.URL}`);
        return {
            status: "completed",
            success: true,
            error: null,
        };
    }
    catch (error) {
        console.error("❌ Navigation failed:", error);
        return {
            status: "completed",
            success: false,
            error: String(error),
        };
    }
}
function getStorageData(keys) {
    return new Promise((resolve, reject) => {
        chrome.storage.local.get(keys, (result) => {
            if (chrome.runtime.lastError) {
                reject(chrome.runtime.lastError.message);
            }
            else {
                resolve(result);
            }
        });
    });
}
// Filters and finds the smallest non-zero number from the text content
function universalFilter(texts, slug) {
    const divideBy100Slugs = ["FC", "Scrooge", "Funrize", "NolimitCoins"];
    let smallestNumber = Infinity;
    console.log(`🔢 universalFilter processing ${texts.length} texts for ${slug}`);
    // Track all found numbers for better debugging
    const allFoundNumbers = [];
    texts.forEach((text, index) => {
        console.log(`\n📝 Processing text ${index + 1}/${texts.length}: "${text}"`);
        const cleanedText = text.replace(/,/g, "");
        console.log(`🧹 Cleaned text: "${cleanedText}"`);
        // Special case for SpinPals - ignore numbers with K or M
        if (slug === "SpinPals") {
            // Skip this text if it contains K or M after a number
            if (/\d+(\.\d+)?[KMkm]/.test(cleanedText)) {
                console.log(`⚠️ Skipping text with K/M suffix for SpinPals`);
                return; // Skip this iteration
            }
        }
        const numberMatch = cleanedText.match(/\d+(\.\d+)?/);
        console.log(`🎯 Number match:`, numberMatch?.[0] || "no match");
        if (numberMatch) {
            let number = parseFloat(numberMatch[0]);
            console.log(`📊 Parsed number: ${number}`);
            if (divideBy100Slugs.includes(slug)) {
                const originalNumber = number;
                number /= 100;
                console.log(`📉 Divided by 100 for ${slug}: ${originalNumber} → ${number}`);
            }
            // Add to all found numbers for debugging
            allFoundNumbers.push(number);
            if (number !== 0 && number < smallestNumber) {
                console.log(`✅ New smallest number found: ${number}`);
                smallestNumber = number;
            }
            else {
                console.log(`❌ Number ${number} rejected: ${number === 0
                    ? "is zero"
                    : "not smaller than current smallest (" + smallestNumber + ")"}`);
            }
        }
    });
    // Log all found numbers for better debugging
    if (allFoundNumbers.length > 0) {
        console.log(`📊 All numbers found for ${slug}:`, allFoundNumbers);
        console.log(`📊 Sorted numbers:`, [...allFoundNumbers].sort((a, b) => a - b));
    }
    else {
        console.log(`⚠️ No numbers found in any text for ${slug}`);
    }
    if (smallestNumber === Infinity) {
        console.log(`❌ No valid non-zero numbers found for ${slug}.`);
        return null;
    }
    else if (smallestNumber > 1200) {
        console.log(`❌ Non-zero number too large: ${smallestNumber}`);
        return null;
    }
    else {
        // Use fixed precision to standardize the output format
        // But avoid using toPrecision() and parseFloat in sequence, which can create rounding errors
        // Instead, use toFixed() to limit decimal places consistently
        const finalNumber = Number(smallestNumber.toFixed(4));
        console.log(`✨ Final smallest non-zero number for ${slug}: ${finalNumber}`);
        return finalNumber;
    }
}
// Simplified version of sendValueUpdate - just forwards the value to the background script
async function sendValueUpdate(accountId, newValue) {
    // Basic validation
    if (!accountId) {
        console.log("❌ Missing accountId in sendValueUpdate");
        return false;
    }
    if (newValue === null || newValue === undefined || newValue === 0) {
        console.log(`⚠️ Invalid value (${newValue}) for ${accountId}`);
        return false;
    }
    console.log(`📤 Sending value update for ${accountId}: ${newValue}`);
    try {
        // Simple direct message to background script
        return new Promise((resolve) => {
            chrome.runtime.sendMessage({
                action: "UPDATE_VALUE",
                accountId,
                value: newValue,
            }, (response) => {
                // Handle potential message port closure
                if (chrome.runtime.lastError) {
                    console.log(`ℹ️ ${chrome.runtime.lastError.message} - Value ${newValue} was found for ${accountId}`);
                    // Consider it a success anyway since we found a value
                    resolve(true);
                    return;
                }
                const success = response?.success || false;
                console.log(success
                    ? `✅ Value update accepted for ${accountId}: ${newValue}`
                    : `❌ Value update rejected for ${accountId}: ${newValue}`);
                resolve(success);
            });
        });
    }
    catch (error) {
        console.error(`❌ Error sending value update: ${error}`);
        return false;
    }
}
// Universal function to find text content based on various selectors
async function universalFindText(options) {
    const { element, attribute, selectorText, textContents, slug, parentElement, parentAttribute, parentSelectorText, } = options;
    console.log(`🔍 Starting value extraction for ${slug}`);
    // Create a cache key for this specific search
    const cacheKey = `${slug}_${element || ""}_${attribute || ""}_${selectorText || ""}_${parentElement || ""}_${parentAttribute || ""}_${parentSelectorText || ""}`;
    // Check if we have a cached result for this exact search
    const cachedResult = window.sessionStorage.getItem(cacheKey);
    if (cachedResult) {
        try {
            const parsedResult = JSON.parse(cachedResult);
            const timestamp = parsedResult.timestamp || 0;
            // Use cache if it's less than 30 seconds old
            if (Date.now() - timestamp < 10000) {
                console.log(`📊 Using cached result for ${slug}`);
                // Process the cached texts
                const finalSum = universalFilter(parsedResult.texts, slug);
                if (finalSum !== null) {
                    console.log(`💰 Final extracted value from cache for ${slug}: ${finalSum}`);
                    try {
                        await sendValueUpdate(slug, finalSum);
                    }
                    catch (error) {
                        console.warn(`⚠️ Error during cached value update for ${slug}:`, error);
                    }
                    return finalSum;
                }
            }
        }
        catch (e) {
            console.warn(`⚠️ Error parsing cached result:`, e);
        }
    }
    let texts = [];
    const MAX_ELEMENTS_TO_PROCESS = 100; // Limit to prevent processing too many elements
    // Build selectors once to avoid repeated string operations
    let selector = "";
    let parentSelector = "";
    // Prioritize search strategies from most specific to least specific
    const searchStrategies = [];
    // Strategy 1: Parent-child query (most specific)
    if (parentElement && parentAttribute && parentSelectorText) {
        parentSelector = `${parentElement}[${parentAttribute}*="${parentSelectorText}"]`;
        searchStrategies.push(async () => {
            console.log(`🔍 Using parent-child selector: "${parentSelector}" > ${element || "*"}`);
            const parents = document.querySelectorAll(parentSelector);
            if (parents.length === 0)
                return false;
            console.log(`📊 Found ${parents.length} parent elements`);
            let foundTexts = false;
            // Process only a limited number of parents
            const parentsToProcess = Array.from(parents).slice(0, 10);
            for (const parentEl of parentsToProcess) {
                const children = parentEl.querySelectorAll(element || "*");
                if (children.length === 0)
                    continue;
                console.log(`📊 Found ${children.length} child elements in parent`);
                // Process only a limited number of children
                const childrenToProcess = Array.from(children).slice(0, MAX_ELEMENTS_TO_PROCESS);
                for (const el of childrenToProcess) {
                    // Special case for Spree - check aria-label
                    if (slug === "Spree") {
                        const ariaLabel = el.getAttribute("aria-label");
                        if (ariaLabel) {
                            texts.push(ariaLabel);
                            console.log(`📝 Extracted aria-label text: "${ariaLabel}"`);
                            foundTexts = true;
                            continue;
                        }
                    }
                    // Default behavior - use textContent
                    const text = el.textContent?.trim() || "";
                    // Only filter by textContents if provided
                    if (!textContents ||
                        text.toLowerCase().includes(textContents.toLowerCase())) {
                        texts.push(text);
                        console.log(`📝 Extracted text content: "${text}"`);
                        foundTexts = true;
                    }
                }
                // Early return if we found enough texts
                if (texts.length >= 10)
                    break;
            }
            return foundTexts;
        });
    }
    // Strategy 2: Attribute selector (specific)
    if (element && attribute && selectorText) {
        selector = `${element}[${attribute}*="${selectorText}"]`;
        searchStrategies.push(async () => {
            console.log(`🔍 Using attribute selector: "${selector}"`);
            const elements = document.querySelectorAll(selector);
            if (elements.length === 0)
                return false;
            console.log(`📊 Found ${elements.length} elements`);
            // Process only a limited number of elements
            const elementsToProcess = Array.from(elements).slice(0, MAX_ELEMENTS_TO_PROCESS);
            let foundTexts = false;
            for (const el of elementsToProcess) {
                // Special case for Spree - check aria-label
                if (slug === "Spree") {
                    const ariaLabel = el.getAttribute("aria-label");
                    if (ariaLabel) {
                        texts.push(ariaLabel);
                        console.log(`📝 Extracted aria-label text: "${ariaLabel}"`);
                        foundTexts = true;
                        continue;
                    }
                }
                // Default behavior - use textContent
                const text = el.textContent?.trim() || "";
                // Only filter by textContents if provided
                if (!textContents ||
                    text.toLowerCase().includes(textContents.toLowerCase())) {
                    texts.push(text);
                    console.log(`📝 Extracted text content: "${text}"`);
                    foundTexts = true;
                }
                // Early return if we found enough texts
                if (texts.length >= 10)
                    break;
            }
            return foundTexts;
        });
    }
    // Strategy 3: Simple element selector (least specific)
    if (element) {
        searchStrategies.push(async () => {
            console.log(`🔍 Using simple element selector: "${element}"`);
            const elements = document.querySelectorAll(element);
            if (elements.length === 0)
                return false;
            console.log(`📊 Found ${elements.length} elements`);
            // Process only a limited number of elements
            const elementsToProcess = Array.from(elements).slice(0, MAX_ELEMENTS_TO_PROCESS);
            let foundTexts = false;
            for (const el of elementsToProcess) {
                // Special case for Spree - check aria-label
                if (slug === "Spree") {
                    const ariaLabel = el.getAttribute("aria-label");
                    if (ariaLabel) {
                        texts.push(ariaLabel);
                        console.log(`📝 Extracted aria-label text: "${ariaLabel}"`);
                        foundTexts = true;
                        continue;
                    }
                }
                // Default behavior - use textContent
                const text = el.textContent?.trim() || "";
                // Only filter by textContents if provided
                if (!textContents ||
                    text.toLowerCase().includes(textContents.toLowerCase())) {
                    texts.push(text);
                    console.log(`📝 Extracted text content: "${text}"`);
                    foundTexts = true;
                }
                // Early return if we found enough texts
                if (texts.length >= 10)
                    break;
            }
            return foundTexts;
        });
    }
    // Execute search strategies in order until one succeeds
    let foundTexts = false;
    for (const strategy of searchStrategies) {
        foundTexts = await strategy();
        if (foundTexts)
            break;
    }
    // Cache the results for future use
    if (texts.length > 0) {
        try {
            window.sessionStorage.setItem(cacheKey, JSON.stringify({
                texts,
                timestamp: Date.now(),
            }));
        }
        catch (e) {
            console.warn(`⚠️ Error caching results:`, e);
        }
    }
    // Process the collected texts
    console.log(`📊 All texts found for ${slug}:`, texts);
    const finalSum = universalFilter(texts, slug);
    // If no valid value was found, don't send a null value update
    // This will ensure the collection is marked as failed
    if (finalSum === null) {
        console.warn(`⚠️ No valid value found for ${slug}, skipping value update`);
        return null;
    }
    console.log(`💰 Final extracted value for ${slug}: ${finalSum}`);
    try {
        // Try to update the value, but don't let a failure here prevent success
        console.log(`🔄 Attempting to send value update for ${slug}: ${finalSum}`);
        await sendValueUpdate(slug, finalSum);
    }
    catch (error) {
        // Log the error but continue - we found a valid value
        console.warn(`⚠️ Error during value update for ${slug}, but value was found:`, error);
    }
    // Return the value that was found, regardless of whether the update succeeded
    console.log(`✅ Value ${finalSum} successfully found for ${slug}`);
    return finalSum;
}
// Update the message listener to handle the new DOM stability check
chrome.runtime.onMessage.addListener((message, sender, sendResponse) => {
    console.log("📨 Content script received message:", message);
    // Add ping handler for content script detection
    if (message.action === "ping") {
        console.log("📍 Ping received, responding...");
        sendResponse({ status: "alive" });
        return true; // Keep the message channel open
    }
    // Add resetActionState handler
    if (message.action === "resetActionState") {
        console.log("🔄 Resetting action state...");
        actionInProgress = false;
        currentAction = null;
        // Clear all active actions
        activeActions.clear();
        sendResponse({ success: true });
        return true; // Keep the message channel open
    }
    if (message.action === "executeAction") {
        // Quick response to acknowledge receipt
        console.log("📩 Execute action received:", message.data);
        sendResponse({ received: true, success: true });
        // Process the action asynchronously
        (async () => {
            const actionDetails = message.data;
            const actionId = actionDetails?.id || `action_${Date.now()}`;
            // Check if this specific action is already in progress
            if (activeActions.has(actionId)) {
                console.warn(`⚠️ Action ${actionId} already in progress`);
                chrome.runtime.sendMessage({
                    type: "ACTION_RESULT",
                    id: actionId,
                    error: "This action is already in progress",
                });
                return;
            }
            console.log("🚀 Starting action:", actionDetails.action, "ID:", actionId);
            // Mark this specific action as in progress
            activeActions.set(actionId, true);
            // For backward compatibility, also set the global flags
            currentAction = actionDetails.action;
            actionInProgress = true;
            try {
                const actionResult = await executeAction(actionDetails);
                console.log("✅ Action completed:", actionDetails.action, actionResult);
                chrome.runtime.sendMessage({
                    type: "ACTION_RESULT",
                    id: actionId,
                    result: actionResult,
                });
            }
            catch (error) {
                console.error("❌ Action failed:", actionDetails.action, error);
                chrome.runtime.sendMessage({
                    type: "ACTION_RESULT",
                    id: actionId,
                    error: error instanceof Error ? error.message : String(error),
                });
            }
            finally {
                // Remove this specific action from tracking
                activeActions.delete(actionId);
                // Only reset global flags if no other actions are in progress
                if (activeActions.size === 0) {
                    actionInProgress = false;
                    currentAction = null;
                }
            }
        })();
        return true; // Keep the message channel open
    }
    // Return false for unhandled messages
    return false;
});
// Initialize on page load
document.addEventListener("DOMContentLoaded", () => {
    // console.log("✅ Content script initialized on DOMContentLoaded");
});
// Also initialize immediately if document is already loaded
if (document.readyState !== "loading") {
    // console.log("✅ Content script initialized (document already loaded)");
}
// Add a more robust URL change detection system
let lastUrl = window.location.href;
// console.log("📍 Initial URL:", lastUrl);

/******/ })()
;
//# sourceMappingURL=automationSequence.js.map